changeset 1226:3407fc016dc6 draft

also accept numbers without dot/decimals for parsing, fixes transaction filter row
author Wladimir J. van der Laan <laanwj@gmail.com>
date Wed, 27 Jul 2011 20:49:14 +0200
parents 6f35013311c1
children d49bc3240f3c
files src/qt/bitcoinunits.cpp
diffstat 1 files changed, 23 insertions(+), 7 deletions(-) [+]
line wrap: on
line diff
--- a/src/qt/bitcoinunits.cpp
+++ b/src/qt/bitcoinunits.cpp
@@ -119,17 +119,33 @@
 
 bool BitcoinUnits::parse(int unit, const QString &value, qint64 *val_out)
 {
-    if(!valid(unit))
-        return false; // Refuse to parse invalid unit
+    if(!valid(unit) || value.isEmpty())
+        return false; // Refuse to parse invalid unit or empty string
     int num_decimals = decimals(unit);
     QStringList parts = value.split(".");
-    if(parts.size() != 2 || parts.at(1).size() > num_decimals)
-        return false; // Max num decimals
+
+    if(parts.size() > 2)
+    {
+        return false; // More than one dot
+    }
+    QString whole = parts[0];
+    QString decimals;
+
+    if(parts.size() > 1)
+    {
+        decimals = parts[1];
+    }
+    if(decimals.size() > num_decimals)
+    {
+        return false; // Exceeds max precision
+    }
     bool ok = false;
-    QString str = parts[0] + parts[1].leftJustified(num_decimals, '0');
-    if(str.size()>18)
-        return false; // Bounds check
+    QString str = whole + decimals.leftJustified(num_decimals, '0');
 
+    if(str.size() > 18)
+    {
+        return false; // Longer numbers will exceed 63 bits
+    }
     qint64 retvalue = str.toLongLong(&ok);
     if(val_out)
     {